package org.datanucleus.test;

import java.util.*;
import org.junit.*;
import javax.jdo.*;

import static org.junit.Assert.*;
import mydomain.model.*;
import org.datanucleus.util.NucleusLogger;

public class SimpleTest
{
    @Test
    public void testMemberClob()
    {
        NucleusLogger.GENERAL.info(">> test START");
        PersistenceManagerFactory pmf = JDOHelper.getPersistenceManagerFactory("MyTest");

        String value1 = "12345678901234567890";
        String value2 = "56789012345678901234";

        PersistenceManager pm = pmf.getPersistenceManager();
        Transaction tx = pm.currentTransaction();
        try
        {
            tx.begin();

            A a = new A(1, "First");
            a.setIdentifier(value1);
            pm.makePersistent(a);

            tx.commit();
        }
        catch (Throwable thr)
        {
            NucleusLogger.GENERAL.error(">> Exception in test", thr);
            fail("Failed test : " + thr.getMessage());
        }
        finally 
        {
            if (tx.isActive())
            {
                tx.rollback();
            }
            pm.close();
        }
        pmf.getDataStoreCache().evictAll();

        pm = pmf.getPersistenceManager();
        tx = pm.currentTransaction();
        try
        {
            tx.begin();

            A a = pm.getObjectById(A.class, 1);
            NucleusLogger.GENERAL.info(">> A retrieved : " + a);
            NucleusLogger.GENERAL.info(">>     identifier=" + a.getIdentifier());
            assertEquals("CLOB member (inserted) is not the same", value1, a.getIdentifier());

            // Update the CLOB
            NucleusLogger.GENERAL.info(">> Updating A.identifier");
            a.setIdentifier(value2);

            tx.commit();
        }
        catch (Throwable thr)
        {
            NucleusLogger.GENERAL.error(">> Exception in test", thr);
            fail("Failed test : " + thr.getMessage());
        }
        finally 
        {
            if (tx.isActive())
            {
                tx.rollback();
            }
            pm.close();
        }
        pmf.getDataStoreCache().evictAll();

        pm = pmf.getPersistenceManager();
        tx = pm.currentTransaction();
        try
        {
            tx.begin();

            A a = pm.getObjectById(A.class, 1);
            NucleusLogger.GENERAL.info(">> A retrieved : " + a);
            NucleusLogger.GENERAL.info(">>     identifier=" + a.getIdentifier());
            assertEquals("CLOB member (updated) is not the same", value2, a.getIdentifier());

            tx.commit();
        }
        catch (Throwable thr)
        {
            NucleusLogger.GENERAL.error(">> Exception in test", thr);
            fail("Failed test : " + thr.getMessage());
        }
        finally 
        {
            if (tx.isActive())
            {
                tx.rollback();
            }
            pm.close();
        }

        pmf.close();
        NucleusLogger.GENERAL.info(">> test END");
    }

    @Test
    public void testCollectionSimpleClob()
    {
        NucleusLogger.GENERAL.info(">> test START");
        PersistenceManagerFactory pmf = JDOHelper.getPersistenceManagerFactory("MyTest");

        String value1 = "12345678901234567890";
        String value2 = "56789012345678901234";

        PersistenceManager pm = pmf.getPersistenceManager();
        Transaction tx = pm.currentTransaction();
        try
        {
            tx.begin();

            B b = new B(1, "First");
            b.getData().add(value1);
            pm.makePersistent(b);

            tx.commit();
        }
        catch (Throwable thr)
        {
            NucleusLogger.GENERAL.error(">> Exception in test", thr);
            fail("Failed test : " + thr.getMessage());
        }
        finally 
        {
            if (tx.isActive())
            {
                tx.rollback();
            }
            pm.close();
        }
        pmf.getDataStoreCache().evictAll();

        pm = pmf.getPersistenceManager();
        tx = pm.currentTransaction();
        try
        {
            tx.begin();

            B b = pm.getObjectById(B.class, 1);
            NucleusLogger.GENERAL.info(">> B retrieved : " + b);
            Collection<String> data = b.getData();
            assertEquals("Number of data elements is wrong", 1, data.size());
            for (String data_row : data)
            {
                assertEquals("B.data element value is wrong", value1, data_row);
            }

            // Update the CLOBs
            NucleusLogger.GENERAL.info(">> Updating b.data");
            b.getData().remove(value1);
            b.getData().add(value2);

            tx.commit();
        }
        catch (Throwable thr)
        {
            NucleusLogger.GENERAL.error(">> Exception in test", thr);
            fail("Failed test : " + thr.getMessage());
        }
        finally 
        {
            if (tx.isActive())
            {
                tx.rollback();
            }
            pm.close();
        }
        pmf.getDataStoreCache().evictAll();

        pm = pmf.getPersistenceManager();
        tx = pm.currentTransaction();
        try
        {
            tx.begin();

            B b = pm.getObjectById(B.class, 1);
            NucleusLogger.GENERAL.info(">> B retrieved : " + b);
            Collection<String> data = b.getData();
            assertEquals("Number of data elements is wrong", 1, data.size());
            for (String data_row : data)
            {
                assertEquals("B.data element value is wrong", value2, data_row);
            }

            tx.commit();
        }
        catch (Throwable thr)
        {
            NucleusLogger.GENERAL.error(">> Exception in test", thr);
            fail("Failed test : " + thr.getMessage());
        }
        finally 
        {
            if (tx.isActive())
            {
                tx.rollback();
            }
            pm.close();
        }

        pmf.close();
        NucleusLogger.GENERAL.info(">> test END");
    }

    @Test
    public void testCollectionSerialisedClob()
    {
        NucleusLogger.GENERAL.info(">> test START");
        PersistenceManagerFactory pmf = JDOHelper.getPersistenceManagerFactory("MyTest");

        String value1 = "12345678901234567890";
        String value2 = "56789012345678901234";

        PersistenceManager pm = pmf.getPersistenceManager();
        Transaction tx = pm.currentTransaction();
        try
        {
            tx.begin();

            C c = new C(1, "First");
            c.getData().add(value1);
            pm.makePersistent(c);

            tx.commit();
        }
        catch (Throwable thr)
        {
            NucleusLogger.GENERAL.error(">> Exception in test", thr);
            fail("Failed test : " + thr.getMessage());
        }
        finally 
        {
            if (tx.isActive())
            {
                tx.rollback();
            }
            pm.close();
        }
        pmf.getDataStoreCache().evictAll();

        pm = pmf.getPersistenceManager();
        tx = pm.currentTransaction();
        try
        {
            tx.begin();

            C c = pm.getObjectById(C.class, 1);
            NucleusLogger.GENERAL.info(">> C retrieved : " + c);
            Collection<String> data = c.getData();
            assertEquals("Number of data elements is wrong", 1, data.size());
            for (String data_row : data)
            {
                assertEquals("C.data element value is wrong", value1, data_row);
            }

            // Update the CLOBs
            NucleusLogger.GENERAL.info(">> Updating c.data");
            c.getData().remove(value1);
            c.getData().add(value2);

            tx.commit();
        }
        catch (Throwable thr)
        {
            NucleusLogger.GENERAL.error(">> Exception in test", thr);
            fail("Failed test : " + thr.getMessage());
        }
        finally 
        {
            if (tx.isActive())
            {
                tx.rollback();
            }
            pm.close();
        }
        pmf.getDataStoreCache().evictAll();

        pm = pmf.getPersistenceManager();
        tx = pm.currentTransaction();
        try
        {
            tx.begin();

            C c = pm.getObjectById(C.class, 1);
            NucleusLogger.GENERAL.info(">> C retrieved : " + c);
            Collection<String> data = c.getData();
            assertEquals("Number of data elements is wrong", 1, data.size());
            for (String data_row : data)
            {
                assertEquals("C.data element value is wrong", value2, data_row);
            }

            tx.commit();
        }
        catch (Throwable thr)
        {
            NucleusLogger.GENERAL.error(">> Exception in test", thr);
            fail("Failed test : " + thr.getMessage());
        }
        finally 
        {
            if (tx.isActive())
            {
                tx.rollback();
            }
            pm.close();
        }

        pmf.close();
        NucleusLogger.GENERAL.info(">> test END");
    }
}
